'use strict';

const URL = require('url');
const htmlparser = require('../modules/htmlparser2');
const Promise = require('../modules/bluebird');
const http = require('http');
const https = require('https');
const assert = require('assert');

class Scraper {


  static _get(protocol, url) {
    assert(protocol === http || protocol === https);
  }

  static _format_url(url) {
    if (url.indexOf('http://') === 0) {
      return url;
    }
    if (url.indexOf('https://') === 0) {
      return url;
    }
    return 'http://' + url;
  }

  static _get_protocol(url) {
    const parsed = URL.parse(url);
    if (parsed.protocol === 'http:') {
      return http;
    }
    if (parsed.protocol === 'https:') {
      return https;
    }
    if (!parsed.protocol) {
      return http;
    }
    return null;
  }

  static _handleRedirect(url, res, nRedirects, cb) {
    const location = res.headers.location;
    if (!location) {
      cb('Nothing to redirect');
      return;
    }
    const newUrl = URL.resolve(url, location);
    this._scrape(newUrl, nRedirects--, cb);
  }

  static _handleOK(url, res, cb) {
    assert(url);
    let insideForm = false;
    const parsed = URL.parse(url);
    const urls = [];
    const forms = [];
    const parser = new htmlparser.Parser({
      onopentag: (name, attrs) => {
	if (insideForm) {
	  const form = forms[forms.length - 1];
	  const name = attrs.name;
	  if (name && name.trim().length > 0) {
	    const value = attrs.value;
	    const arg = {};
	    arg[name] = value;
	    form.args.push(arg);
	  }
	}
	if (name.toLowerCase() === 'form') {
	  insideForm = true;
	  assert(url);
	  const action = attrs.action || '';
	  forms.push({
	    method: attrs.method,
	    action: URL.resolve(url, action),
	    args: []
	  });
	  return;
	}
	if (name.toLowerCase() === 'a') {
	  const href = attrs.href;
	  if (!href) {
	    return;
	  }
	  
	  const resolved = URL.resolve(url, href);
	  const parsed1 = URL.parse(resolved);
	  if (parsed1.host !== parsed.host) {
	    return;
	  }
	  if (parsed1.protocol !== parsed.protocol) {
	    return;
	  }
	  if (parsed1.path === parsed.path) {
	    return;
	  }
	  if (!parsed1.pathname.endsWith('.php')) {
	    return;
	  }
	  for (let i = 0; i < urls.length; i++) {
	    if (urls[i].pathname === parsed1.pathname) {
	      return;
	    }
	  }
	  console.log('pushing: %s', parsed1.pathname);
	  urls.push(parsed1);
	  return;	  
	}
      },
      onclosetag: name => {
	if (name.toLowerCase() === 'form') {
	 insideForm = false;
	}
      }
    }, {
      decodeEntities: true
    });
    res.on('data', d => {
      parser.write(d.toString());
      //console.log(d.toString());
    });
    res.on('end', () => {
      parser.end();
      console.log(forms);
      for (let i = 0; i < urls.length; i++) {
	urls[i] = URL.format(urls[i]);
      }
      cb(null, {
	urls: urls,
	forms: forms
      });
    });

  }

  static _scrape(url, nRedirects, cb) {
    console.log('calling _scrape: %s %s', url, nRedirects);
    if (nRedirects < 0) {
      cb('too much redirects');
      return;
    }
    url = this._format_url(url);
    console.log('scraping: %s, %d', url, nRedirects);
    const protocol = this._get_protocol(url);

    const req = protocol.get(url, res => {
      const headers = res.headers;
      const status = res.statusCode;
      if (status >= 200 && status < 300) {
	this._handleOK(url, res, cb);
	return;
      }
      if (status >= 300 && status < 400) {
	this._handleRedirect(url, res, nRedirects, cb);
	return;
      }
      cb('invalid satusCode: %d', status);
    });

    req.on('error', err => {
      cb(err);
    });
  }

  static scrape(url, cb) {
    this._scrape(url, 3, cb);
  }

}

Promise.promisifyAll(Scraper, {
  filter: function() {
    return true;
  }
});

module.exports = Scraper;
