'use strict';

const URL = require('url');
const querystring = require('querystring');
const assert = require('assert');
const Promise = require('../modules/bluebird');
const http = require('http');
const https = require('https');


class HostChecker {
  constructor() {
    Promise.promisifyAll(this, {
      filter: function() {
        return true;
      }
    });
  }

  _isHttp(host) {
    if (host.indexOf('http://') === 0) {
      return true;
    }
    return false;
  }

  _sendLoginRequest(host, user, pass, nTry, cb) {
    assert(nTry > 0);
    let getter = https;
    if (this._isHttp(host)) {
      getter = http;
    }

    let url = host + 'wp-login.php';
    let parsedUrl = URL.parse(url);
    let postData = {
      'log' : user,
      'pwd' : pass,
      'wp-submit' : 'Log in',
      'redirect_to' : host + 'wp-admin/'
    };
    postData = querystring.stringify(postData);
    let port = 80;
    if (getter === https) {
      port = 443;
    };
    if (parsedUrl.port) {
      port = parsedUrl.port;
    }
    let HEADERS = {'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Encoding':'gzip, deflate', 'Accept-Language':'en-US,en;q=0.8,ka;q=0.6', 'Cache-Control':'max-age=0', 'Connection':'keep-alive', 'Content-Type':'application/x-www-form-urlencoded', 'User-Agent':"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36"};
    let options = {
      hostname: parsedUrl.hostname,
      //    hostname: ip,
      port: port,
      path: parsedUrl.path,
      method: 'POST',
      headers: HEADERS
    };
    options.headers['Content-Length'] = postData.length;
    options.headers['Host'] = parsedUrl.hostname;

    let response = null;

    let req = getter.request(options, res => {
      let status = res.statusCode;
      res.setEncoding('utf8');
      response = res;
      let content = '';
      console.log('response received: %d', status);
      console.log(res.rawHeaders);
      res.on('data', chunk => {
        console.log('ondata');
        content += chunk;
      });
      res.on('end', () => {
        console.log('onend');
        cb(null, {
          status: status,
          headers: res.rawHeaders,
          content: content
        });
        cb = function() {};
      });

    });

    req.on('error', (e) => {
      nTry--;
      if (nTry > 0) {
        this._sendLoginRequest(host, user, pass, nTry, cb);
        return;
      }
      console.log('eminem');
      console.log(host);
      console.log(e);
      cb(null, null);
      //      cb(e);
      cb = function() {};
    });

    req.write(postData);
    req.end();

  }

  _formatHost(host) {
    let res = host;
    if (host.indexOf('http://') === 0) {
      res = host;
    } else if (host.indexOf('https://') === 0) {
      res = host;
    } else {
      res = 'http://' + host;
    }

    if (res[res.length - 1] === '/') {
      return res;
    } else {
      return res + '/';
    }
  }

  _isLoggedIn(source) {
    if (source.indexOf('type="password"') !== -1) {
      return false;
    }
    if (source.indexOf("type='password'") !== -1) {
      return false;
    }
    // change
    if (source.indexOf('var ajaxurl = \'/wp/wp-admin/admin-ajax.php\'') !== -1) {
      return true;
    }
    return undefined;
  }


  check(host, user, pass, timeout, cb) {
    host = this._formatHost(host);
    this._sendLoginRequestAsync(host, user, pass, 1)
      .then(resp => {
        let result = null;
        if (!resp) {
	  return false;
        }
        if (resp.status >= 400) {
          result = false;
        }
        if (result !== false) {
          result = this._isLoggedIn(resp.content);
        }
        if (result === undefined) {
          let parsedHeaders = '';
	  const headers = resp.headers;
          for (let i = 0; i < headers.length; i++) {
            let header = headers[i];
            console.log(header);
            if (header.indexOf('Set-Cookie') !== -1) {
              i++;
              if (i >= headers.length) {
                break;
              }
              header = headers[i].toLowerCase();
              if (header.indexOf('path=/wp-content/plugins') !== -1) {
                if (header.indexOf(user + '%7c') !== -1) {
                  result = true;
                }
                break;
              }

            }
          }
        }
	if (result === undefined) {
	  result = false;
	}
	assert(result !== undefined);
	return result;
      })
      .then(result => {
	cb(null, result);
      })
      .timeout(timeout)
      .catch(Promise.TimeoutError, err => {
	cb(null, false);
      });
  }

}

module.exports = HostChecker;
