'use strict';

const Promise = require('../modules/bluebird');
const config = require('./config');
const Utils = require('./utils');
const assert = require('assert');
const net = require('net');
const Socks5 = require('./socks5');

const instances = [];
const IPPairs = new Map();

class Proxy {
  constructor(bindIP, bindPort) {
    this._bindIP = bindIP;
    this._bindPort = bindPort;
    this._server = net.createServer(this._onConnection.bind(this));
    this._nTries = 3;
    Promise.promisifyAll(this, {
      filter: function() {
        return true;
      }
    });
    this._server.on('error', err => {
      if (this._onListenError) {
        this._onListenError(err);
      }
    });
    instances.push(this);
  }

  _listen(ip, port, nTries, cb) {
    if (this._listening) {
      cb(null, this._bindPort);
    }
    if (nTries > 0) {
      assert(!port);
      port = Utils.getRandomInt(config.portLow, config.portHigh);
    }
    assert(port);
    this._server.listen(port, ip, () => {
      this._listening = true;
      this._bindPort = port;
      cb(null, port);
      cb = Function.prototype;
    });
    console.log('ntries: %d', nTries);
    this._onListenError = (err) => {
      if (nTries > 1) {
        this._listen(ip, undefined, nTries - 1, cb);
        return;
      }
      cb(err);
      cb = Function.prototype;
    };
  }

  listen(cb) {
    let port;
    let nTries;
    if (!this._bindPort) {
      port = undefined;
      nTries = 10;
    } else {
      port = this._bindPort;
      nTries = 0;
    }
    this._listen(this._bindIP, port, nTries, (err, port) => {
      if (err) {
        cb(err);
        return;
      }
      this._bindPort = port;
      cb(null, port);
    });

  }

  delete() {
    const index = instances.indexOf(this);
    if (index === -1) {
      return;
    }
    instances.splice(index, 1);
    this._server.close();
  }

  _chooseBindIP(defaultBindIP, currentExternalIP) {
    assert(defaultBindIP);
    assert(currentExternalIP);
    if (defaultBindIP !== '0.0.0.0') {
      return defaultBindIP;
    }
    const result = IPPairs.get(currentExternalIP);
    assert(result);
    return result;
  }

  _onConnection(sock) {
//    const bindIP = this._chooseBindIP(this._bindIP, sock.localAddress);
//    new Socks5(sock, null, bindIP);
    new Socks5(sock, null, this._bindIP, IPPairs);
  }

  static setIPPair(externalIP, internalIP) {
    IPPairs.set(externalIP, internalIP);
  }

  static find(ip, port) {
    if (instances.length === 0) {
      return null;
    }
    if (!port) {
      return instances[0];
    }

    for (const item of instances) {
      if (item._bindIP !== ip) {
        continue;
      }
      if (item._bindPort !== port) {
        continue;
      }
      return item;
    }
    return null;
  }


}


module.exports = Proxy;
