'use strict';

const EventEmitter = require('events');
const Socks5 = require('./socks5');
const assert = require('assert');
const net = require('net');
const Utils = require('./utils');
const Promise = require('../modules/bluebird');
const config = require('./config');

const states = {
  PENDING: 0,			// connection being created
  FRESH: 1,			// waiting for client
  SOCKS: 2			// client connected
};

const instances = [];

class ProxyCon {
  constructor(bindIP, peerIP, peerPort) {
    this._bindIP = bindIP;
    this._peerIP = peerIP;
    this._peerPort = peerPort;
    this._state = states.PENDING;
    this._emitter = new EventEmitter();
    this._connect(() => {
    });
  }

  on(event, cb) {
    this._emitter.on(event, cb);
  }

  getState() {
    return this._state;
  }

  _connect(cb) {
    let couldNotConnect;
    let socket;
    const onError = err => {
      if (this._state !== states.PENDING) {
	return;
      }
      clearTimeout(couldNotConnect);
      cb('peerUnreachable');
      cb = Function.prototype;
    };

    const onClose = () => {
      if (this._state === states.PENDING) {
	clearTimeout(couldNotConnect);
	cb('peerUnreachable');
	cb = Function.prototype;
      }
      this._emitter.emit('close');
    };

    const onTimeout = () => {
      if (this._state === states.FRESH) {
	socket.end();
	socket.destroy();
      }
      assert(this._state !== states.PENDING);
    };

    couldNotConnect = setTimeout(() => {
      socket.end();
      socket.destroy();
    }, 20 * 1000);

    const startSocks5 = initialRecData => {
      this._state = states.SOCKS;
      this._lastSocksCreateionTime = new Date();
      socket.setTimeout(0);
      this._emitter.emit('start');
      const socks5 = new Socks5(socket, initialRecData, this._bindIP);
    };

    this._state = states.PENDING;
    socket = net.connect({
      port: this._peerPort,
      host: this._peerIP,
      localAddress: this._bindIP
    }, () => {
      cb();
      cb = Function.prototype;
      this._emitter.emit('connect');
      this._state = states.FRESH;
      socket.once('data', d => {
	console.log('starting socks5');
	startSocks5(d);
      });
      //startSocks5('');
    });

    socket.on('close', onClose.bind(this));
    socket.on('timeout', onTimeout.bind(this));
    socket.on('error', onError.bind(this));
    socket.setTimeout(60 * 1000); // todo: +- random
  }
};

class ReverseProxyBack {
  constructor(bindIP, peerIP, peerPort) {
    this._bindIP = bindIP;
    this._peerIP = peerIP;
    this._peerPort = peerPort;
    Promise.promisifyAll(this, {
      filter: function() {
	return true;
      }
    });

    this._peerIsReachable = undefined;
    this._cons = [];
    this._manageProxies();
    setInterval(() => {
      this._manageProxies();
    }, 100);
    instances.push(this);    
  }

  _countAll() {
    let nPending = 0;
    let nFresh = 0;
    let nSocks = 0;
    for (const con of this._cons) {
      if (con.getState() === states.PENDING) {
	nPending++;
      }
      if (con.getState() === states.FRESH) {
	nFresh++;
      }
      if (con.getState() === states.SOCKS) {
	nSocks++;
      }
    }

    return {
      nPending,
      nFresh,
      nSocks
    };
  };

  _manageProxies() {
    const counted = this._countAll();
    console.log(counted);
    const nPending = counted.nPending;
    const nFresh = counted.nFresh;
    const nSocks = counted.nSocks;

    let nNeeded = nSocks - (nFresh + nPending);
    const nNeededRaw = nNeeded;
    if (nNeeded <= 0 && nFresh === 0 && nPending === 0) {
      nNeeded = 1;
    }
    if (nNeeded > 10) {
      nNeeded = 10;
    }
    console.log('%d needed', nNeeded);
    for (let i = 0; i < nNeeded; i++) {
      const con = new ProxyCon(this._bindIP, this._peerIP, this._peerPort);
      let connected = false;
      con.on('start', () => {
	this._manageProxies();
      });
      con.on('connect', () => {
	connected = true;
	if (this._peerIsReachable === undefined) {
	  this._peerIsReachable = true;
	}
      });
      con.on('close', () => {
	if (this._peerIsReachable === undefined) {
	  this._peerIsReachable = false;
	}
	const index = this._cons.indexOf(con);
	assert(index !== -1);
	this._cons.splice(index, 1);
      });
      this._cons.push(con);
    }
  }

  isUp(cb) {
    if (this._peerIsReachable === undefined) {
      setTimeout(() => {
	this.isUp(cb);
      }, 1000);
      return;
    }
    process.nextTick(() => {
      cb(null, this._peerIsReachable);
    });
  }

  delete() {
    // todo: cleanup this bitch
    const index = instances.indexOf(this);
    if (index === -1) {
      return;
    }
    instances.splice(index, 1);
  }

  static find(ip, peerIP, peerPort) {
    for (const item of instances) {
      if (item._bindIP !== ip) {
	continue;
      }
      if (item._peerIP !== peerIP) {
	continue;
      }
      if (item._peerPort !== peerPort) {
	continue;
      }
      return item;
    }
    return null;
  }

}

module.exports = ReverseProxyBack;
